package mypackage;

import org.faceless.pdf2.*;
import org.faceless.pdf2.viewer2.*;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.Date;
import java.text.SimpleDateFormat;
import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.event.UndoableEditEvent;

/**
 * This is a demonstration feature which will create a custom
 * stamp whereever the user clicks on the page.
 */
public class MyStamp extends ToggleViewerWidget implements PagePanelInteractionListener {

    // MyStamp is a "ToggleViewerWidget", so we need to set which group of
    // toggle buttons its in. The "Mode" group includes the ZoomToFit, TextSelection,
    // AnnotationAdd and similar features so that seems like the right one.
    //
    private static final String GROUP = "Mode";

    private String name, stamptext;

    public MyStamp() {
        super("MyStamp", GROUP);
        // Note the path to the image is absolute - recent versions of the
        // PDF Library (> 2.11.19) can use relative paths too
        setButton(GROUP, "/mypackage/resources/exclamation.png", "My Custom Stamp");
    }

    public void initialize(PDFViewer viewer) {
        super.initialize(viewer);

        // Use the "name" and "stamptext" properties if they're defined.
        name = getFeatureProperty(viewer, "name");
        if (name == null) {
            name = viewer.getCurrentUser();
        }
        stamptext = getFeatureProperty(viewer, "stamptext");
        if (stamptext == null) {
            stamptext = "UNACCEPTABLE!";
        }
    }

    /**
     * This method is part of ToggleViewerWidget, and it's called whenever this
     * feature get's selected or deselected. We use it to ensure our listener
     * is on the viewport only when we're active.
     */
    public void updateViewport(DocumentViewport viewport, boolean selected) {
        if (selected) {
            viewport.addPagePanelInteractionListener(this);
        } else {
            viewport.removePagePanelInteractionListener(this);
        }
    }

    /**
     * This method is the method for our PagePanelListener interface, and it's called
     * whenever the mouse moves, clicks etc. on the PagePanel.
     */
    public void pageAction(PagePanelInteractionEvent event) {

        // Theoretically we're only being called when this feature is selected - that's
        // what we're doing in updateViewport. However double checking never hurts.
        if (isSelected() && event.getType() == "mouseClicked") {

            // Create the stamp annotation
            final AnnotationStamp stamp = new AnnotationStamp();
            PDFCanvas canvas = createCanvas();
            stamp.setCanvas("mystamp", canvas);
            stamp.setAuthor(name);

            // Center the stamp on the mouse-click and add to page.
            // We set the size of the annotation to the size of the
            // canvas, but this isn't a requirement - stamps can
            // stretch and squash as needed.
            Point2D point = event.getPoint();
            final PDFPage page = event.getPage();
            float x = (float)point.getX();
            float y = (float)point.getY();
            float w2 = canvas.getWidth() / 2;
            float h2 = canvas.getHeight() / 2;
            stamp.setRectangle(x-w2, y-h2, x+w2, y+h2);
            stamp.setPage(page);

            // Add an "Undoable Edit" to the DocumentPanel
            final DocumentPanel docpanel = event.getPagePanel().getDocumentPanel();
            docpanel.fireUndoableEditEvent(new UndoableEditEvent(docpanel, new AbstractUndoableEdit() {
                public String getPresentationName() {
                    return stamptext;
                }
                public void undo() {
                    super.undo();  // Don't forget this!
                    stamp.setPage(null);
                }
                public void redo() {
                    super.redo();  // Don't forget this!
                    stamp.setPage(page);
                }
            }));
        }
    }

    /**
     * Create a PDFCanvas for the stamp. This is all PDF Library stuff, there's nothing
     * specific to the viewer here.
     */
    private PDFCanvas createCanvas() {
        int w = 150;    // Arbitrary
        int h = 60;
        PDFCanvas canvas = new PDFCanvas(w, h);
        PDFStyle style = new PDFStyle();

        // Draw a line around the canvas and fill it with white
        style.setFillColor(Color.white);
        style.setLineColor(Color.red);
        style.setLineWeighting(2);
        canvas.setStyle(style);
        canvas.drawRoundedRectangle(2, 2, w-2, h-2, 6);

        // Create a LayoutBox and add the header text
        LayoutBox box = new LayoutBox(w);
        PDFStyle headerstyle = new PDFStyle();
        headerstyle.setFillColor(Color.red);
        headerstyle.setFont(new StandardFont(StandardFont.HELVETICABOLD), 16);
        headerstyle.setTextAlign(PDFStyle.TEXTALIGN_CENTER);
        box.addTextNoBreak(stamptext, headerstyle, null);
        box.addLineBreak(headerstyle);

        // Add the body text to the stamp
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        String now = df.format(new Date());
        PDFStyle bodystyle = new PDFStyle(headerstyle);
        bodystyle.setFont(new StandardFont(StandardFont.HELVETICAOBLIQUE), 11);
        box.addText(name + " - " + now, bodystyle, null);

        // Draw the LayoutBox to the canvas and return it
        box.flush();
        float texth = box.getHeight();
        canvas.drawLayoutBox(box, 0, (h+texth) / 2);
        canvas.flush();
        return canvas;
    }

}
